/*
 *	TXX Interface Definitions
 *	Copyright (C) 1997-2003 Nils M Holm. All rights reserved.
 *	See the file LICENSE in the T3X documentation directory
 *	($T3XDIR/doc) for conditions of use.
 */

#define EXTERN extern

#ifdef MAIN
 #undef EXTERN
 #define EXTERN
#endif

#define UCHAR   unsigned char
#define USHORT  unsigned short
#define UINT    unsigned int
#define ULONG   unsigned long

#ifdef __STDC__
 #ifndef COHERENT
  #define ANSIPROTO
 #endif
#else
 #ifdef __TURBOC__
  #define ANSIPROTO
 #endif
#endif

#ifdef __FreeBSD__
 #ifdef __i386__
  #define HOST32
  #define NL_IS_LF
 #else
  /* pre-3.0 FreeBSDs? */
  #ifdef i386
   #define HOST32
   #define NL_IS_LF
  #endif
  #ifdef __alpha__
   #define HOST64
   #define NL_IS_LF
  #endif
 #endif
#endif

#ifdef __NetBSD__
 #ifdef __alpha__
  #define HOST64
  #define NL_IS_LF
 #endif
 #ifdef i386
  #define HOST32
  #define NL_IS_LF
 #endif
 #ifdef mc68000
  #define HOST32
  #define NL_IS_LF
 #endif
#endif  

#ifdef COHERENT
 #ifdef _I386
  #define HOST32
  #define NL_IS_LF
  #define NEED_RENAME
  #define void char
  #include <sys/types.h>
 #endif
#endif

#ifdef __linux__
 #ifdef __i386__
  #define HOST32
  #define NL_IS_LF
 #endif
#endif  

#ifdef dos
 #define HOST16
 #define NL_IS_CRLF
 #include <dos.h>
 #ifdef __TURBOC__
  #define off_t	ULONG
  /* OOPS! open() and friends translate LF<-->CRLF */
  #define creat(X,Y)	_creat((X),0)
  #define open		_open
  #define close		_close
  #define read		_read
 #endif
#endif

#ifndef HOST16
 #ifndef HOST32
  #ifndef HOST64
   #error "Host machine word size undefined!"
  #endif
 #endif
#endif

#ifdef HOST16
 #define CELL	short
 #define UCELL	USHORT
 #define BPW	2
 #define WORDCAST	(short)
 #ifdef __TURBOC__
  /* TurboC port of TXX uses large model */
  #define XCELL		long
  #define XUCELL	ULONG
 #else
  #define XCELL		short
  #define XUCELL	USHORT
 #endif
#endif

#ifdef HOST32
 #ifdef LIBRARY
  #define CELL	int
  #define UCELL	UINT
  #define BPW	4
 #else
  #define CELL	short
  #define UCELL	USHORT
  #define BPW	2
 #endif
 #define WORDCAST	(int)
 #define XCELL	int
 #define XUCELL	UINT
#endif

#ifdef HOST64
 #ifdef LIBRARY
  #define CELL	long
  #define UCELL	ULONG
  #define BPW	8
 #else
  #define CELL	short
  #define UCELL	USHORT
  #define BPW	2
 #endif
 #define WORDCAST	(long)
 #define XCELL	long
 #define XUCELL	ULONG
#endif

#ifndef LIBRARY
 #define ARG(C,N)	Stack[Sp+(C)-(N)]
 #define UARG(C,N)	Ustack[Sp+(C)-(N)]
 #define ARGP(C,N)	(char *) &Data[Ustack[Sp+(C)-(N)]]
 #define UARGP(C,N)	&Data[Ustack[Sp+(C)-(N)]]
#endif

#ifdef LIBRARY
 #define S0()			(__self)
 #define S1(a0)			(__self,a0)
 #define S2(a0,a1)		(__self,a1,a0)
 #define S3(a0,a1,a2)		(__self,a2,a1,a0)
 #define S4(a0,a1,a2,a3)	(__self,a3,a2,a1,a0)
 #define S5(a0,a1,a2,a3,a4)	(__self,a4,a3,a2,a1,a0)
 #define S6(a0,a1,a2,a3,a4,a5)	(__self,a5,a4,a3,a2,a1,a0)
 #define S7(a0,a1,a2,a3,a4,a5,a6)	\
				(__self,a6,a5,a4,a3,a2,a1,a0)
#else
  #define S0()			()
  #define S1(a0)		(a0)
  #define S2(a0,a1)		(a1,a0)
  #define S3(a0,a1,a2)		(a2,a1,a0)
  #define S4(a0,a1,a2,a3)	(a3,a2,a1,a0)
  #define S5(a0,a1,a2,a3,a4)	(a4,a3,a2,a1,a0)
  #define S6(a0,a1,a2,a3,a4,a5)	(a5,a4,a3,a2,a1,a0)
  #define S7(a0,a1,a2,a3,a4,a5,a6)	\
				(a6,a5,a4,a3,a2,a1,a0)
#endif

EXTERN USHORT	*Ustack;
EXTERN short	*Stack;
EXTERN USHORT	Sp;
EXTERN UCHAR	*Data;
EXTERN short	Acc;
EXTERN char	**Argvec;
EXTERN int	Argcount;

#define MAXINFPROC	64
struct infproc {
	char	*name;
	XCELL	(*proc)();
};

#ifdef MAIN

void	libinit();

void	X_bpw(), X_close(), X_cvalist(), X_getarg(), X_getenv(),
	X_memcomp(), X_memcopy(), X_memfill(), X_memscan(),
	X_newline(), X_open(), X_read(), X_remove(),
	X_rename(),
	X_seek(), X_sys(), X_write();

void	(*Primitives[])() = {

	/*---TXX-PRIMITIVES-BEGIN---*/
	X_sys,
	X_bpw,
	X_memcopy,
	X_memcomp,
	X_memfill,
	X_memscan,
	X_getarg,
	X_getenv,
	X_open,
	X_read,
	X_write,
	X_newline,
	X_seek,
	X_close,
	X_rename,
	X_remove,
	X_cvalist,
	/*---TXX-PRIMITIVES-END---*/

	0
};

#endif	/* MAIN */
